@@ -67,6 +67,11 @@ SickSafetyscannersRos::SickSafetyscannersRos()
   m_field_service_server =
     m_nh.advertiseService("field_data", &SickSafetyscannersRos::getFieldData, this);
 
+  m_config_metadata_server =
+    m_nh.advertiseService("config_metadata", &SickSafetyscannersRos::getConfigMetadata, this);
+  m_status_overview_server =
+    m_nh.advertiseService("status_overview", &SickSafetyscannersRos::getStatusOverview, this);
+
   // Diagnostics for frequency
   m_diagnostic_updater.setHardwareID(m_communication_settings.getSensorIp().to_string());
 
@@ -79,7 +84,9 @@ SickSafetyscannersRos::SickSafetyscannersRos()
   m_diagnostic_updater.add("State", this, &SickSafetyscannersRos::sensorDiagnostics);
 
   m_device = std::make_shared<sick::SickSafetyscanners>(
-    boost::bind(&SickSafetyscannersRos::receivedUDPPacket, this, _1), &m_communication_settings);
+    boost::bind(&SickSafetyscannersRos::receivedUDPPacket, this, _1),
+    &m_communication_settings,
+    m_interface_ip);
   m_device->run();
   readTypeCodeSettings();
 
@@ -89,6 +96,9 @@ SickSafetyscannersRos::SickSafetyscannersRos()
   }
 
   m_device->changeSensorSettings(m_communication_settings);
+  m_device->requestConfigMetadata(m_communication_settings, config_meta_data);
+  m_device->requestFirmwareVersion(m_communication_settings, firmware_version);
+
   m_initialised = true;
   ROS_INFO("Successfully launched node.");
 }
@@ -172,6 +182,13 @@ bool SickSafetyscannersRos::readParameters()
   }
   m_communication_settings.setHostIp(host_ip_adress);
 
+  std::string interface_ip_adress = "0.0.0.0";
+  if (!m_private_nh.getParam("interface_ip", interface_ip_adress))
+  {
+    ROS_WARN("Using default interface IP: %s", interface_ip_adress.c_str());
+  }
+  m_interface_ip = boost::asio::ip::address_v4::from_string(interface_ip_adress);
+
   int host_udp_port = 0;
   if (!m_private_nh.getParam("host_udp_port", host_udp_port))
   {
@@ -293,9 +310,13 @@ void SickSafetyscannersRos::sensorDiagnostics(
   diagnostic_status.addf("Version major version", "%u", header.version_major_version);
   diagnostic_status.addf("Version minor version", "%u", header.version_minor_version);
   diagnostic_status.addf("Version release", "%u", header.version_release);
+  diagnostic_status.addf("Firmware version", "%s", firmware_version.getFirmwareVersion().c_str());
   diagnostic_status.addf("Serial number of device", "%u", header.serial_number_of_device);
   diagnostic_status.addf(
     "Serial number of channel plug", "%u", header.serial_number_of_channel_plug);
+  diagnostic_status.addf("App checksum", "%X", config_meta_data.getAppChecksum());
+  diagnostic_status.addf("Overall checksum", "%X", config_meta_data.getOverallChecksum());
+
   diagnostic_status.addf("Channel number", "%u", header.channel_number);
   diagnostic_status.addf("Sequence number", "%u", header.sequence_number);
   diagnostic_status.addf("Scan number", "%u", header.scan_number);
@@ -324,6 +345,18 @@ void SickSafetyscannersRos::sensorDiagnostics(
   {
     diagnostic_status.summary(diagnostic_msgs::DiagnosticStatus::ERROR, "Device error");
   }
+  else if (state.application_error)
+  {
+    diagnostic_status.summary(diagnostic_msgs::DiagnosticStatus::ERROR, "Application error");
+  }
+  else if (state.contamination_error)
+  {
+    diagnostic_status.summary(diagnostic_msgs::DiagnosticStatus::ERROR, "Contamination error");
+  }
+  else if (state.contamination_warning)
+  {
+    diagnostic_status.summary(diagnostic_msgs::DiagnosticStatus::WARN, "Contamination warning");
+  }
   else
   {
     diagnostic_status.summary(diagnostic_msgs::DiagnosticStatus::OK, "OK");
@@ -820,5 +853,86 @@ bool SickSafetyscannersRos::getFieldData(sick_safetyscanners::FieldData::Request
   return true;
 }
 
+bool SickSafetyscannersRos::getConfigMetadata(sick_safetyscanners::ConfigMetadata::Request& req,
+                                              sick_safetyscanners::ConfigMetadata::Response& res)
+{
+  static_cast<void>(req);
+  sick::datastructure::ConfigMetadata config_metadata;
+  m_device->requestConfigMetadata(m_communication_settings, config_metadata);
+
+  res.app_checksum = getCheckSumString(config_metadata.getAppChecksum());
+  res.integrity_hash.clear();
+  std::vector<uint32_t> integrity_hash = config_metadata.getIntegrityHash();
+  for (const auto& hash : integrity_hash)
+  {
+    res.integrity_hash.push_back(hash);
+  }
+  res.modification_time_date = config_metadata.getModificationTimeDate();
+  res.modification_time_time = config_metadata.getModificationTimeTime();
+  res.modification_time  = getDateString(res.modification_time_date, res.modification_time_time);
+  res.overall_checksum   = getCheckSumString(config_metadata.getOverallChecksum());
+  res.transfer_time_date = config_metadata.getModificationTimeDate();
+  res.transfer_time_time = config_metadata.getModificationTimeTime();
+  res.transfer_time      = getDateString(res.transfer_time_date, res.transfer_time_time);
+  res.version_c_version  = config_metadata.getVersionCVersion();
+  res.version_major_version_number = config_metadata.getVersionMajorVersionNumber();
+  res.version_minor_version_number = config_metadata.getVersionMinorVersionNumber();
+  res.version_release_number       = config_metadata.getVersionReleaseNumber();
+
+  return true;
+}
+
+bool SickSafetyscannersRos::getStatusOverview(sick_safetyscanners::StatusOverview::Request& req,
+                                              sick_safetyscanners::StatusOverview::Response& res)
+{
+  static_cast<void>(req);
+
+  sick::datastructure::StatusOverview status_overview;
+  m_device->requestStatusOverview(m_communication_settings, status_overview);
+
+  res.version_c_version            = status_overview.getVersionCVersion();
+  res.version_major_version_number = status_overview.getVersionMajorVersionNumber();
+  res.version_minor_version_number = status_overview.getVersionMinorVersionNumber();
+  res.version_release_number       = status_overview.getVersionReleaseNumber();
+
+  res.device_state                = status_overview.getDeviceState();
+  res.config_state                = status_overview.getConfigState();
+  res.application_state           = status_overview.getApplicationState();
+  res.current_time_power_on_count = status_overview.getCurrentTimePowerOnCount();
+
+  res.current_time_date = status_overview.getCurrentTimeDate();
+  res.current_time_time = status_overview.getCurrentTimeTime();
+  res.current_time      = getDateString(res.current_time_date, res.current_time_time);
+
+  res.error_info_code = status_overview.getErrorInfoCode();
+
+  res.error_info_time_date = status_overview.getErrorInfoDate();
+  res.error_info_time_time = status_overview.getErrorInfoTime();
+  res.error_info_time      = getDateString(res.error_info_time_date, res.error_info_time_time);
+
+  return true;
+}
+
+std::string SickSafetyscannersRos::getCheckSumString(uint32_t checksum)
+{
+  std::stringstream ss;
+  ss << "0x" << std::hex << (checksum & 0xFF) << ((checksum & 0xFF00) >> 8)
+     << ((checksum & 0xFF0000) >> 16) << ((checksum & 0xFF000000) >> 24);
+  return ss.str();
+}
+
+std::string SickSafetyscannersRos::getDateString(uint32_t days_since_1972, uint32_t milli_seconds)
+{
+  std::time_t t = static_cast<std::time_t>((730 /* = days from Jan 1 1970 to Jan 1 1972*/
+                                            + days_since_1972) *
+                                             24 * 3600 +
+                                           milli_seconds * 0.001);
+  char buffer[40];
+  std::string retval;
+  strftime(buffer, 40, "%F %X", gmtime(&t));
+  retval = buffer;
+  return retval;
+}
+
 
 } // namespace sick
